// src/breakpoint-helper.ts
function bph(config) {
  let breakpoints = {};
  if (typeof config === "string" && config === "meta") {
    breakpoints = _getBpsFromMeta();
  } else if (typeof config === "string" && config === "custom") {
    breakpoints = _getBpsFromCustomProps();
  } else {
    breakpoints = config;
  }
  if (Object.keys(breakpoints).length === 0) {
    throw new Error(`No breakpoints defined`);
  }
  function _getBpsFromMeta() {
    const el = document.createElement("meta");
    el.classList.add("breakpoint-helper");
    document.getElementsByTagName("head")[0].appendChild(el);
    let fontFamily = window.getComputedStyle(el).getPropertyValue("font-family");
    if (fontFamily.length <= 0)
      return {};
    fontFamily = fontFamily.replace(/'|"/g, "").split("&");
    return fontFamily.reduce((obj, elem) => {
      const [name, value] = elem.split("=");
      obj[name] = value;
      return obj;
    }, {});
  }
  function _getBpsFromCustomProps() {
    const sheets = Array.from(document.styleSheets).filter(
      (sheet) => sheet?.href?.indexOf(window.location.origin) !== -1
    );
    const rules = [...sheets].reduce((acc, sheet) => {
      Array.from(sheet.cssRules).forEach((rule) => {
        if (rule instanceof CSSStyleRule && rule.selectorText === ":root") {
          const css = rule.cssText.split("{")[1].replace("}", "").split(";");
          css.forEach((dec) => {
            const [prop, val] = dec.split(":");
            if (prop.indexOf("--bph-") !== -1) {
              acc[prop.replace("--bph-", "").trim()] = val.trim();
            }
          });
        }
      });
      return acc;
    }, {});
    return rules;
  }
  function _matchAll(keys, useMax = false) {
    const matches = [];
    keys.forEach((bp) => {
      if (isMatching(bp, useMax)) {
        matches.push(bp);
      }
    });
    return useMax ? matches : matches.reverse();
  }
  function getBreakpoints() {
    return breakpoints;
  }
  function getMediaQuery(name, useMax = false) {
    if (Array.isArray(name)) {
      const [min2, max] = name;
      return `${getMediaQuery(min2)} and ${getMediaQuery(max, true)}`;
    }
    const min = breakpoints[name];
    if (typeof min === "undefined") {
      throw new Error(`"${name}" does not seem to be a breakpoint name`);
    }
    if (useMax) {
      const number = parseFloat(min);
      const unit = min.replace(number, "");
      const substract = unit === "em" ? 0.0635 : 1;
      return `(max-width: ${number - substract}${unit})`;
    }
    return `(min-width: ${min})`;
  }
  function isMatching(name, useMax = false) {
    return window.matchMedia(getMediaQuery(name, useMax)).matches;
  }
  function listen(options, callback) {
    let mq = null;
    const opts = { name: "", useMax: false, immediate: true };
    if (typeof options === "string" || Array.isArray(options)) {
      opts.name = options;
    } else {
      opts.name = options.name;
      opts.useMax = options.useMax || false;
      opts.immediate = typeof options.immediate === "undefined" ? true : options.immediate;
    }
    function on() {
      if (typeof opts.name === "string" || Array.isArray(opts.name)) {
        mq = window.matchMedia(getMediaQuery(opts.name));
      } else {
        mq = window.matchMedia(getMediaQuery(opts.name, opts.useMax));
      }
      if (opts.immediate)
        callback(mq);
      mq.addEventListener("change", callback);
    }
    on();
    function off() {
      if (mq) {
        mq.removeEventListener("change", callback);
        mq = null;
      }
    }
    return { on, off };
  }
  function listenAll(callback, options = {}) {
    const keys = Object.keys(breakpoints);
    let listeners = [];
    let bps = keys;
    if (keys.length === 0)
      return { on: () => {
      }, off: () => {
      } };
    const { useMax, immediate, listenTo } = options;
    const opts = {
      useMax: useMax || false,
      immediate: typeof immediate === "undefined" ? true : immediate
    };
    if (listenTo) {
      bps = listenTo.sort((a, b) => {
        return parseInt(breakpoints[a], 10) - parseInt(breakpoints[b], 10);
      });
    }
    const cb = () => {
      callback(_matchAll(bps, opts.useMax));
    };
    function on() {
      bps.forEach((bp) => {
        const listener = listen(
          { name: bp, useMax: opts.useMax, immediate: false },
          cb
        );
        listeners.push(listener);
      });
      if (opts.immediate)
        cb();
    }
    on();
    function off() {
      if (listeners.length) {
        listeners.forEach((listener) => listener.off());
        listeners = [];
      }
    }
    return { on, off };
  }
  return {
    getBreakpoints,
    getMediaQuery,
    isMatching,
    listen,
    listenAll
  };
}
var breakpoint_helper_default = bph;
export {
  breakpoint_helper_default as default
};
//# sourceMappingURL=breakpoint-helper.js.map
